# Node.js Interop for Dart
![Dart](https://img.shields.io/badge/dart-%3E%3D1.24-brightgreen.svg) ![Node.js](https://img.shields.io/badge/node-%3E%3D8.9-brightgreen.svg) ![Release](https://img.shields.io/pub/v/nodejs_interop.svg) ![License](https://img.shields.io/badge/license-MIT-blue.svg) ![Coverage](https://coveralls.io/repos/github/cedx/nodejs-interop.dart/badge.svg) ![Build](https://travis-ci.org/cedx/nodejs-interop.dart.svg)

This package provides [Node.js](https://nodejs.org) interoperability for [Dart](https://www.dartlang.org).

## Requirements
- The latest [Dart SDK](https://www.dartlang.org) and [Pub](https://pub.dartlang.org) versions.
- The latest [Node.js](https://nodejs.org) and [npm](https://www.npmjs.com) versions.

If you plan to play with the sources, you will also need the latest [Grinder](http://google.github.io/grinder.dart) version.

## Installing via [Pub](https://pub.dartlang.org)

### 1. Depend on it
Add this to your package's `pubspec.yaml` file:

```yaml
dependencies:
  nodejs_interop: *
```

### 2. Install it
Install this package and its dependencies from a command prompt:

```shell
$ pub get
```

### 3. Import it
Now in your [Dart](https://www.dartlang.org) code, you can use:

```dart
import 'dart:js';
import 'package:nodejs_interop/nodejs_interop.dart';
```

## Usage

### The [globals](https://nodejs.org/api/globals.html) module
It provides objects that are available globally:

```dart
print(dirname); // "/path/to"
print(filename); // "app.dart.js"
print(module.id); // "/path/to/app.dart.js"

print(process.arch); // "x64"
print(process.platform); // "linux"
```

### The [`assert`](https://nodejs.org/api/assert.html) module
This module provides a simple set of assertion tests that can be used to test invariants:

```dart
AssertModule _assert = require('assert');

_assert.ok(true); // Returns normally.
_assert.ok(false); // Throws a `JSAssertionError`.
```

### The [`events`](https://nodejs.org/api/events.html) module
This module provides the `EventEmitter` class:

```dart
// Required: the `EventEmitter` class doesn't exist before this call.
loadLibrary('events');

var listener = allowInterop((data) => print('A "eventName" event occurred: ${data['foo']}'));
var emitter = new EventEmitter()..on('eventName', listener);
emitter.emit('eventName', new JsObject.jsify({'foo': 'bar'}));
```

### The [`os`](https://nodejs.org/api/os.html) module
This module provides a number of operating system-related utility methods:

```dart
OSModule os = require('os');

print(os.homedir()); // "/home/user"
print(os.tmpdir()); // "/tmp"
```

### The [`path`](https://nodejs.org/api/path.html) module
This module provides utilities for working with file and directory paths:

```dart
PathModule path = require('path');

print(path.basename('/foo/bar/baz.txt')); // "baz.txt"
print(path.dirname('/foo/bar/baz.txt')); // "/foo/bar"
```

## Unit tests
In order to run the tests, you must install the [Node.js](https://nodejs.org) test dependencies by running the following command:

```shell
$ NODE_ENV=development npm install
```

Then, you must tell the test runtime where are located the newly installed [npm](https://www.npmjs.com) packages by setting the `NODE_PATH` environment variable to the path of the local `node_modules` folder:

```shell
$ export NODE_PATH="$PWD/node_modules"
```

Finally, you can launch the test runner from the command prompt:

```shell
$ pub run test
```

## See also
- [API reference](https://cedx.github.io/nodejs-interop.dart)
- [Code coverage](https://coveralls.io/github/cedx/nodejs-interop.dart)
- [Continuous integration](https://travis-ci.org/cedx/nodejs-interop.dart)

## License
[Node.js Interop for Dart](https://github.com/cedx/nodejs-interop.dart) is distributed under the MIT License.
