/// Provides file, socket, HTTP, and other I/O support for [Node.js](https://nodejs.org) applications.
library nodejs_interop.io;

import 'dart:async';
import 'dart:convert';
import 'dart:io' as io;
import 'dart:js';
import 'package:collection/collection.dart';
import 'package:file/file.dart' as f;
import 'package:path/path.dart' as p;
import 'package:platform/platform.dart' as platform;
import 'js.dart';
import 'node.dart';
import 'npm.dart';

// ignore_for_file: directives_ordering
export 'package:file/file.dart' show
  ErrorCodes,
  FileLock,
  FileMode,
  FileSystemEntityType,
  FileSystemException,
  IOException,
  OSError;

part 'src/io/directory.dart';
part 'src/io/file.dart';
part 'src/io/file_stat.dart';
part 'src/io/file_system.dart';
part 'src/io/file_system_entity.dart';
part 'src/io/file_system_events.dart';
part 'src/io/link.dart';
part 'src/io/io_sink.dart';
part 'src/io/platform.dart';
part 'src/io/random_access_file.dart';

/// A reference to the `fs` module.
final FSModule _fs = loadLibrary<FSModule>('fs');

/// A reference to the `os` module.
final OSModule _os = loadLibrary<OSModule>('os');

/// A reference to the `path` module.
final PathModule _path = loadLibrary<PathModule>('path');
