/// Provides interoperability with [JavaScript](https://developer.mozilla.org/en-US/docs/Web/JavaScript) APIs.
@JS()
library nodejs_interop.js;

import 'dart:async';
import 'dart:js';
import 'dart:js_util';
import 'dart:typed_data';
import 'package:js/js.dart';

part 'src/js/errors.dart';
part 'src/js/json.dart';
part 'src/js/object.dart';
part 'src/js/promise.dart';
part 'src/js/typed_arrays.dart';

/// A function invoked when an operation is finally completed.
typedef void CompletionCallback0([JSError error]);

/// A function invoked with a value when an operation is finally completed.
typedef void CompletionCallback1<T>(JSError error, [T value]);

/// A function invoked with two values when an operation is finally completed.
typedef void CompletionCallback2<T, U>(JSError error, [T value1, U value2]);

/// Converts the specified JavaScript [value] to a Dart object.
dynamic convertToDart(value) {
  if (value == null) return null;
  if (value is bool || value is num || value is DateTime || value is String || value is TypedData) return value;
  if (value is List) return value.map(convertToDart).toList();
  return new Map<String, dynamic>.fromIterable(_JSObject.keys(value), value: (key) => convertToDart(getProperty(value, key)));
}

/// Converts the specified [promise] to a [Future].
Future<T> convertToFuture<T>(Promise<T> promise) {
  var completer = new Completer<T>();
  promise.then(allowInterop(completer.complete), allowInterop(completer.completeError));
  return completer.future;
}
