part of nodejs_interop.io;

/// Provides random access to the data in a file.
abstract class RandomAccessFile implements f.RandomAccessFile {

  /// The file descriptor of the underlying file.
  final int _fd;

  /// Creates a new random access file.
  RandomAccessFile._internal(this._fd, this.path);

  /// The path of the underlying file.
  @override
  final String path;

  /// Closes the file. Returns a [Future] that completes with this [RandomAccessFile] when it has been closed.
  @override
  Future<RandomAccessFile> close() {
    var completer = new Completer<RandomAccessFile>();
    _fs.close(_fd, allowInterop(([error]) {
      if (error == null) completer.complete(this);
      else completer.completeError(new f.FileSystemException(error.message, path));
    }));

    return completer.future;
  }

  /// Synchronously closes the file. Throws a [FileSystemException] if the operation fails.
  @override
  void closeSync() {
    try { _fs.closeSync(_fd); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }
}
