part of nodejs_interop.js;
// ignore_for_file: avoid_classes_with_only_static_members

/// A typed view of a sequence of bytes.
@JS('TypedArray')
abstract class TypedArray<T> {

  /// The byte buffer associated with this object.
  external ByteBuffer get buffer;

  /// The number of elements in this array.
  external int get length;

  /// Creates and returns an iterator of `[index, byte]` pairs from the contents of this array.
  external Iterable<List> entries();

  /// Fills this array with the specified [value]. If the [offset] and [end] are not given, the entire array will be filled.
  external TypedArray<T> fill(T value, [int offset = 0, int end]);

  /// Gets a value indicating whether this array includes the specified [element].
  external bool includes(T element, [int start = 0]);

  /// Returns the first index of [element] in this array.
  ///
  /// Searches the array from index [start] to the end of the array.
  /// Returns -1 if [element] is not found.
  external int indexOf(T element, [int start = 0]);

  /// Creates and returns an iterator of array keys (i.e. indices).
  external Iterable<int> keys();

  /// Returns the last index of [element] in this array.
  ///
  /// Searches the array backwards from index [start] to `0`.
  /// Returns -1 if [element] is not found.
  external int lastIndexOf(T element, [int start = 0]);

  /// Returns a new list containing the elements from [start] inclusive to [end] exclusive.
  external TypedArray<T> slice([int start = 0, int end]);

  /// Creates and returns an iterator of array values (i.e. bytes).
  external Iterable<T> values();
}

/// An array of 8-bit unsigned integers.
@JS('Uint8Array')
class Uint8Array extends TypedArray<int> {

  /// The number of bytes of an element.
  external static int get BYTES_PER_ELEMENT;

  /// The string value of the constructor name.
  external static String get name;
}
