/// Provides interoperability with [`console`](https://nodejs.org/api/console.html) module.
@JS()
library nodejs_interop.node.console;

import 'package:js/js.dart';
import 'package:nodejs_interop/src/node/stream.dart';
import 'package:nodejs_interop/src/node/util.dart';

/// A simple logger with configurable output streams.
@JS('console.Console')
class Console {

  /// Creates a new console by passing one or two writable stream instances.
  ///
  /// [stdout] is a writable stream to print log or info output. [stderr] is used for warning or error output.
  /// If [stderr] is not passed, warning and error output will be sent to [stdout].
  external Console(Writable stdout, [Writable stderr]);

  /// Attempts to clear the console. When `stdout` is not a TTY, this method does nothing.
  external void clear();

  /// Maintains an internal counter specific to [label] and outputs to `stdout` the number of times [count] has been called with the given [label].
  external void count([String label = 'default']);

  /// Resets the internal counter specific to [label].
  external void countReset([String label = 'default']);

  /// Uses [UtilModule.inspect] on [object] and prints the resulting string to `stdout`.
  ///
  /// This function bypasses any custom inspection function defined on [object].
  /// An optional [options] object may be passed to alter certain aspects of the formatted string.
  external void dir(object, [InspectOptions options]);

  /// Prints to `stderr` with newline.
  /// Multiple arguments can be passed, with the first used as the primary message and all additional used as substitution values.
  external void error(data, [arg1, arg2, arg3, arg4, arg5, arg6, arg7]);

  /// Increases indentation of subsequent lines by two spaces.
  /// If one or more labels are provided, those are printed first without the additional indentation.
  external void group([label1, label2, label3, label4, label5, label6, label7, label8]);

  /// An alias for the [group] method.
  external void groupCollapsed();

  /// Decreases indentation of subsequent lines by two spaces.
  external void groupEnd();

  /// An alias for the [log] method.
  external void info(data, [arg1, arg2, arg3, arg4, arg5, arg6, arg7]);

  /// Prints to `stdout` with newline.
  /// Multiple arguments can be passed, with the first used as the primary message and all additional used as substitution values.
  external void log(data, [arg1, arg2, arg3, arg4, arg5, arg6, arg7]);

  /// Starts a timer that can be used to compute the duration of an operation.
  /// Timers are identified by a unique [label].
  external void time([String label = 'default']);

  /// Stops a timer that was previously started by calling [time] and prints the result to `stdout`.
  external void timeEnd([String label = 'default']);

  /// Prints to `stderr` the string `"Trace:"`, followed by the formatted [message] and stack trace to the current position in the code.
  external void trace(message, [arg1, arg2, arg3, arg4, arg5, arg6, arg7]);

  /// An alias for the [error] method.
  external void warn(data, [arg1, arg2, arg3, arg4, arg5, arg6, arg7]);
}
