part of nodejs_interop.node.globals;

/// Indicates that an exception occurred within the program's runtime environment.
/// Typically, these are operational errors that occur when an application violates an operating system constraint
/// such as attempting to read a file that does not exist or when the user does not have sufficient permissions.
@JS()
abstract class JSSystemError implements JSError { // TODO concrete class?

  /// When present, the address to which the connection failed.
  external String get address;
  external set address(String value);

  /// An integer or a string.
  /// The integer is a negative value which corresponds to the error code defined in [`libuv` error handling](http://docs.libuv.org/en/v1.x/errors.html).
  /// In case of a string, it is the same as [code].
  external dynamic get errno;
  external set errno(value);

  /// When present, a relevant invalid pathname.
  external String get path;
  external set path(String value);

  /// When present, the connection's port that is not available.
  external int get port;
  external set port(int value);

  /// The [`syscall`](http://manpages.ubuntu.com/manpages/xenial/man2/syscall.2.html) that failed.
  external String get syscall;
  external set syscall(String value);
}
