/// Provides interoperability with [`del`](https://www.npmjs.com/package/del) package.
@JS()
library nodejs_interop.npm.del;

import 'package:js/js.dart';
import 'package:nodejs_interop/js.dart';

/// The [`del`](https://www.npmjs.com/package/del) module.
@JS()
abstract class DelModule implements Function {

  /// Deletes the file system entities corresponding to the specified globbing [patterns] for files.
  ///
  /// The file [patterns] can be a [String] or a [List] of strings.
  /// Returns a [Promise] that resolves with the paths of the deleted entities.
  external Promise<List<String>> call(patterns, [DeleteOptions options]);

  /// Synchronously deletes the file system entities corresponding to the specified globbing [patterns] for files.
  ///
  /// The file [patterns] can be a [String] or a [List] of strings.
  /// Returns the paths of the deleted entities.
  external List<String> sync(patterns, [DeleteOptions options]);
}

/// Options used to customize the file deletion.
@JS()
@anonymous
class DeleteOptions extends GlobOptions {

  /// Creates a new options object.
  external factory DeleteOptions({
    bool dryRun = false,
    bool force = false,
    String cwd,
    bool dot = false,
    bool mark = false,
    bool nomount = false,
    bool nosort = false,
    String root,
    bool silent = false,
    bool stat = false,
    bool strict = false
  });

  /// Whether to only return the paths that would be deleted, without removing the corresponding file system entities.
  external bool get dryRun;

  /// Whether to allow deleting the current working directory and outside.
  external bool get force;
}

/// Options used to customize the file globbing.
@JS()
@anonymous
class GlobOptions {

  /// Creates a new options object.
  external factory GlobOptions({
    String cwd,
    bool dot = false,
    bool mark = false,
    bool nomount = false,
    bool nosort = false,
    String root,
    bool silent = false,
    bool stat = false,
    bool strict = false
  });

  /// The current working directory in which to search. Defaults to `process.cwd()`.
  external String get cwd;

  /// Include .dot files in normal matches and globstar matches.
  /// Note that an explicit dot in a portion of the pattern will always match dot files.
  external bool get dot;

  /// Add a `/` character to directory matches. Note that this requires additional stat calls.
  external bool get mark;

  /// By default, a pattern starting with a forward-slash will be "mounted" onto the root setting,
  /// so that a valid filesystem path is returned. Set this flag to disable that behavior.
  external bool get nomount;

  /// Don't sort the results.
  external bool get nosort;

  /// The place where patterns starting with `/` will be mounted onto. Defaults to `path.resolve(options.cwd, "/")`.
  external String get root;

  /// When an unusual error is encountered when attempting to read a directory,
  /// a warning will be printed to `stderr`. Set this flag to suppress these warnings.
  external bool get silent;

  /// Set this flag to stat all results.
  /// This reduces performance somewhat, and is completely unnecessary, unless `readdir` is presumed
  /// to be an untrustworthy indicator of file existence.
  external bool get stat;

  /// When an unusual error is encountered when attempting to read a directory,
  /// the process will just continue on in search of other matches. Set this flag to raise an error in these cases.
  external bool get strict;
}
