import 'dart:js';
import 'dart:js_util';
import 'package:nodejs_interop/nodejs_interop.dart';
import 'package:test/test.dart';

/// Tests the features of the functions.
void main() {
  group('convertToDart()', () {
    test('should leave unchanged the supported Dart types', () {
      var now = new DateTime.now();
      expect(convertToDart(null), isNull);
      expect(convertToDart(false), isFalse);
      expect(convertToDart(123), equals(123));
      expect(convertToDart(-123.456), equals(-123.456));
      expect(convertToDart(now), equals(now));
      expect(convertToDart('foo'), equals('foo'));
    });

    test('should convert the supported JavaScript types', () {
      var list = new JsArray();
      expect(convertToDart(list), allOf(isList, isEmpty));
      list.add('foo');
      expect(convertToDart(list), allOf(isList, hasLength(1), contains('foo')));

      var object = newObject();
      expect(convertToDart(object), allOf(isMap, isEmpty));
      setProperty(object, 'foo', 'bar');
      expect(convertToDart(object), allOf(isMap, hasLength(1), containsPair('foo', 'bar')));
    });
  });

  group('convertToFuture()', () {
    test('should return a future that completes with a value when the promise resolves', () async {
      var future = convertToFuture(new Promise(allowInterop((resolve, reject) => resolve('foo'))));
      expect(await future, equals('foo'));
    });

    test('should return a future that completes with an error when the promise rejects', () {
      var future = convertToFuture(new Promise(allowInterop((resolve, reject) => reject(new Exception()))));
      expect(() async => await future, throwsException);
    });
  });
}
