import 'package:nodejs_interop/nodejs_interop.dart';
import 'package:nodejs_interop/npm.dart';
import 'package:test/test.dart';

/// Tests the features of the [ReaddirEnhancedModule] class.
void main() => group('ReaddirEnhancedModule', () {
  final path = loadLibrary<PathModule>('path');
  final readdirEnhanced = loadLibrary<ReaddirEnhancedModule>('readdir-enhanced');

  var testDirs = const ['io', 'js', 'node', 'npm'];
  var testFiles = [path.join('npm', 'os_locale_test.dart'), path.join('npm', 'readdir_enhanced_test.dart')];

  group('.readdirAsync()', () {
    test('should list the sub-directories and files as strings', () async {
      var list = await convertToFuture<List<String>>(readdirEnhanced.readdirAsync('test'));
      for (var entity in testDirs) expect(list, contains(entity), reason: entity);

      list = await convertToFuture<List<String>>(readdirEnhanced.readdirAsync('test', new ReaddirOptions(deep: true)));
      for (var entity in testFiles) expect(list, contains(entity), reason: entity);
    });
  });

  group('.readdirAsyncStat()', () {
    test('should list the sub-directories and files as `Stats`', () async {
      var list = await convertToFuture<List<Stats>>(readdirEnhanced.readdirAsyncStat('test'));
      for (var stats in list) expect(stats.isDirectory() || stats.isFile(), isTrue);
    });
  });

  group('.readdirStream()', () {
    test('should stream the sub-directories and files as strings', () {
      var stream = convertToStream<String>(readdirEnhanced.readdirStream('test'));
      for (var entity in testDirs) expect(stream, emitsThrough(equals(entity)), reason: entity);

      stream = convertToStream<String>(readdirEnhanced.readdirStream('test', new ReaddirOptions(deep: true)));
      for (var entity in testFiles) expect(stream, emitsThrough(equals(entity)), reason: entity);
    }, skip: true); // TODO: bad bad tests! Not working with a single subscription stream.
  });

  group('.readdirStreamStat()', () {
    test('should stream the sub-directories and files as `Stats`', () async {
      var stream = convertToStream<Stats>(readdirEnhanced.readdirStreamStat('test'));
      await for (var stats in stream) expect(stats.isDirectory() || stats.isFile(), isTrue);
    });
  });

  group('.readdirSync()', () {
    test('should synchronously list the sub-directories and files as strings', () {
      var list = readdirEnhanced.readdirSync('test');
      for (var entity in testDirs) expect(list, contains(entity), reason: entity);

      list = readdirEnhanced.readdirSync('test', new ReaddirOptions(deep: true));
      for (var entity in testFiles) expect(list, contains(entity), reason: entity);
    });
  });

  group('.readdirSyncStat()', () {
    test('should synchronously list the sub-directories and files as `Stats`', () {
      var list = readdirEnhanced.readdirSyncStat('test');
      for (var stats in list) expect(stats.isDirectory() || stats.isFile(), isTrue);
    });
  });
});
